"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const url_1 = require("url");
const crypto_1 = require("crypto");
const AvroTypes_enum_1 = require("./AvroTypes.enum");
exports.AvroTypes = AvroTypes_enum_1.AvroTypes;
const AvroLogicalTypes_enum_1 = require("./AvroLogicalTypes.enum");
exports.AvroLogicalTypes = AvroLogicalTypes_enum_1.AvroLogicalTypes;
const md5 = (data) => crypto_1.createHash('md5')
    .update(data)
    .digest('hex');
const firstToUpper = (str) => `${str[0].toUpperCase()}${str.slice(1)}`;
const normalizeName = (fileName) => fileName
    .split('-')
    .map(firstToUpper)
    .join('');
const getName = ($id) => $id.replace(RE_FILENAME, (_, p1, p2) => normalizeName(p2));
const typeMapping = {
    string: AvroTypes_enum_1.AvroTypes.String,
    null: AvroTypes_enum_1.AvroTypes.Null,
    boolean: AvroTypes_enum_1.AvroTypes.Boolean,
    integer: AvroTypes_enum_1.AvroTypes.Integer,
    number: AvroTypes_enum_1.AvroTypes.Number,
};
const RE_SYMBOL = /^[A-Za-z_][A-Za-z0-9_]*$/;
const RE_NAMESPACE = /(.*)\/(v\d)\/.*/i;
const RE_ESCAPE = /([^a-z0-9]+)/gi;
const RE_FILENAME = /.*\/(v\d)\/([a-z-]*).json/i;
const getTopLevelFields = ({ properties, required }, additionalFields = []) => {
    if (!properties) {
        return [...additionalFields];
    }
    return [...convertProperties(properties, required), ...additionalFields];
};
exports.convert = (jsonSchema, options = {}) => {
    if (!jsonSchema) {
        throw new Error('No schema given');
    }
    if (!jsonSchema.$id) {
        throw new Error('No $id provided for schema: https://json-schema.org/understanding-json-schema/basics.html#declaring-a-unique-identifier');
    }
    return {
        namespace: getNamespace(jsonSchema.$id),
        name: getName(jsonSchema.$id),
        type: AvroTypes_enum_1.AvroTypes.Record,
        doc: jsonSchema.description,
        fields: getTopLevelFields(jsonSchema, options.additionalFields),
    };
};
const getNamespace = (id = '') => {
    const url = url_1.parse(id);
    if (!url.host) {
        throw new Error('Every top-level schema should set $id to an absolute URI: https://json-schema.org/understanding-json-schema/structuring.html#id');
    }
    const prefix = url.host.split('.').reverse();
    if (!url.path) {
        return prefix.join('.');
    }
    return prefix
        .concat(url
        .path.replace(RE_NAMESPACE, (_, p1) => p1)
        .replace(RE_ESCAPE, '.')
        .slice(1))
        .join('.');
};
const isComplex = (schema) => {
    return schema.type === 'object';
};
const isArray = (schema) => {
    return schema.type === 'array';
};
const hasEnum = (schema) => {
    return Boolean(schema.enum);
};
const convertProperties = (schema, required = [], parent = '') => {
    return Object.keys(schema).map(item => {
        const isRequired = Boolean(required.find(key => key === item));
        if (isComplex(schema[item])) {
            return convertComplexProperty(item, schema[item], isRequired, parent);
        }
        else if (isArray(schema[item])) {
            return convertArrayProperty(item, schema[item], isRequired, parent);
        }
        else if (hasEnum(schema[item])) {
            return convertEnumProperty(item, schema[item], isRequired);
        }
        return convertProperty(item, schema[item], isRequired);
    });
};
const getComplexType = (simpleName, contents, parent) => {
    const name = parent ? `${simpleName}_${md5(parent)}` : `${simpleName}_record`;
    return {
        type: AvroTypes_enum_1.AvroTypes.Record,
        name,
        fields: convertProperties(contents.properties || {}, contents.required, name),
    };
};
const convertComplexProperty = (name, contents, isRequired = false, item) => {
    const type = getComplexType(name, contents, item);
    return {
        name,
        doc: contents.description || '',
        ...resolveRequiredType(type, isRequired),
    };
};
const convertArrayProperty = (name, contents, isRequired, item) => {
    const type = {
        type: AvroTypes_enum_1.AvroTypes.Array,
        items: isComplex(contents.items)
            ? getComplexType(name, contents.items, item)
            : convertProperty(name, contents.items, true),
    };
    return {
        name,
        doc: contents.description || '',
        ...resolveRequiredType(type, isRequired),
    };
};
const convertEnumProperty = (name, contents, isRequired = false) => {
    const valid = contents.enum.every((symbol) => RE_SYMBOL.test(symbol));
    const type = valid
        ? {
            type: AvroTypes_enum_1.AvroTypes.Enum,
            name: `${name}_enum`,
            symbols: contents.enum,
        }
        : AvroTypes_enum_1.AvroTypes.String;
    const prop = {
        name,
        doc: contents.description || '',
        ...resolveRequiredType(type, isRequired),
    };
    if (contents.hasOwnProperty('default')) {
        prop.default = contents.default;
    }
    return prop;
};
const convertProperty = (name, value, isRequired = false) => {
    const prop = {
        name,
        doc: value.description || '',
    };
    const { type, default: defaultValue } = resolveRequiredType(getTypeForProp(value), isRequired, value.default);
    prop.type = type;
    if (defaultValue !== undefined) {
        prop.default = defaultValue;
    }
    if (value.hasOwnProperty('default')) {
        prop.default = value.default;
    }
    return prop;
};
const getTypeForProp = (value) => {
    if (Array.isArray(value.type)) {
        return value.type.map((type) => typeMapping[type]);
    }
    return typeMapping[value.type];
};
const resolveRequiredType = (type, isRequired, defaultValue) => {
    let calculatedType;
    const isDefaulted = typeof defaultValue !== 'undefined';
    if (isRequired) {
        calculatedType = type;
    }
    else {
        if (isDefaulted) {
            if (Array.isArray(type)) {
                calculatedType = [
                    ...type.filter((k) => k !== AvroTypes_enum_1.AvroTypes.Null),
                    AvroTypes_enum_1.AvroTypes.Null,
                ];
            }
            else {
                calculatedType = [type, AvroTypes_enum_1.AvroTypes.Null];
            }
        }
        else if (Array.isArray(type)) {
            calculatedType = [
                AvroTypes_enum_1.AvroTypes.Null,
                ...type.filter((k) => k !== AvroTypes_enum_1.AvroTypes.Null),
            ];
        }
        else {
            calculatedType = [AvroTypes_enum_1.AvroTypes.Null, type];
        }
    }
    if (calculatedType[0] === AvroTypes_enum_1.AvroTypes.Null) {
        return {
            type: calculatedType,
            default: null,
        };
    }
    if (typeof defaultValue !== 'undefined') {
        return {
            type: calculatedType,
            default: defaultValue,
        };
    }
    return { type: calculatedType };
};
//# sourceMappingURL=index.js.map